==> gsbot-maven-plugin-1.0.0-sources.jar: io/gupshup/maven/Add.java <==
/*
 * Copyright 2017 Gupshup.io. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.gupshup.maven;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.List;
import java.util.Properties;

import org.apache.maven.model.Resource;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;
import org.eclipse.jgit.api.CheckoutCommand;
import org.eclipse.jgit.api.CreateBranchCommand.SetupUpstreamMode;
import org.eclipse.jgit.api.FetchCommand;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.InitCommand;
import org.eclipse.jgit.api.RemoteAddCommand;
import org.eclipse.jgit.transport.URIish;
import org.json.JSONObject;

import io.gupshup.maven.utils.Constants;
import io.gupshup.maven.utils.Constants.SMAPIConsts;
import io.gupshup.maven.utils.SMAPIWrapper;

/**
 * @author Abhishek Nama
 */
@Mojo(name = "add", threadSafe = true, defaultPhase = LifecyclePhase.INITIALIZE, requiresDependencyResolution = ResolutionScope.COMPILE_PLUS_RUNTIME)
public class Add extends AbstractMojo {
    @Parameter(defaultValue = "${project}", readonly = true, required = true)
    private MavenProject project;

    @Parameter(property = "apiKey", required = true)
    private String apiKey;

    @Parameter(property = "botName", required = true)
    private String botName;

    @Override
    public void execute() throws MojoExecutionException {
	try {
	    List<?> resources = project.getResources();
	    if (resources.size() < 1)
		throw new MojoExecutionException("Project resources not found.");

	    Resource resource = (Resource) resources.get(0);
	    File gsCfgPropsFile = new File(resource.getDirectory() + File.separator + Constants.BOT_PROPERTIES);
	    if (!gsCfgPropsFile.exists())
		throw new MojoExecutionException("gsbot-cfg.properties not found in src" + File.separator + "main"
			+ File.separator + "resources");

	    Properties gsCfgProps = new Properties();
	    gsCfgProps.load(new FileInputStream(gsCfgPropsFile));

	    getLog().info("");
	    getLog().info("------------------------------------------------------------------------ ");
	    getLog().info("Adding bot " + botName + " for " + project.getName());
	    getLog().info("------------------------------------------------------------------------ ");
	    getLog().info("");
	    
	    /*getLog().info("Checking bot type...");

	    JSONObject getBotMetaResp = SMAPIWrapper.getBotMeta(apiKey, botName);
	    if (getBotMetaResp.getInt("status") != 200)
		throw new MojoExecutionException(getBotMetaResp.getString("body"));

	    JSONObject getBotMetaRespBody = new JSONObject(getBotMetaResp.getString("body"));
	    if (!(getBotMetaRespBody.has("function") && getBotMetaRespBody.getJSONObject("function").has("lang")
		    && getBotMetaRespBody.getJSONObject("function").getString("lang").equals("java"))) {
		throw new MojoExecutionException("Only java bots are supported.");
	    }

	    getLog().info("Bot type is Java.");
	    getLog().info("");*/

	    getLog().info("Fetching bot details...");

	    JSONObject getBotSpecResp = SMAPIWrapper.getBotSpec(apiKey, botName);
	    if (getBotSpecResp.getInt("status") != 200)
		throw new MojoExecutionException(getBotSpecResp.getString("body"));

	    String botKey = new JSONObject(getBotSpecResp.getString("body")).getString("uuid");

	    getLog().info("Configuring gsbot-cfg.properties");

	    gsCfgProps.setProperty("gupshup.dev.apikey", apiKey);
	    gsCfgProps.setProperty("gupshup.bot.name", botName);
	    gsCfgProps.setProperty("gupshup.bot.key", botKey);
	    gsCfgProps.store(new FileOutputStream(gsCfgPropsFile), null);

	    // Creating bot repository if not yet created.
	    SMAPIWrapper.createGit(botKey);

	    getLog().info("Synching with bot repository.");

	    String userId = SMAPIWrapper.getUserId(apiKey);
	    if (userId != null) {
		String gitUrl = SMAPIConsts.GIT_URL.replace("__USERID", userId).replace("__APIKEY", apiKey)
			.replace("__BOTKEY", botKey);

		try {
		    cloneBotRepo(gitUrl);
		} catch (Exception e) {
		    getLog().info("");
		    if (e.getMessage() != null)
			getLog().warn(e.getMessage());
		    else
			getLog().warn("Failed to sync with bot repository.");
		}
	    } else {
		getLog().info("");
		getLog().error("Failed to sync with bot repository.");
	    }

	    getLog().info("");
	    getLog().info("Bot added successfully!");
	    getLog().info("");
	} catch (Exception e) {
	    throw new MojoExecutionException("", e);
	}
    }

    private void cloneBotRepo(String gitUrl) throws Exception {
	InitCommand initCommand = Git.init().setDirectory(project.getBasedir());
	Git git = initCommand.call();

	RemoteAddCommand remoteAddCommand = git.remoteAdd();
	remoteAddCommand.setName("origin");
	remoteAddCommand.setUri(new URIish(gitUrl));
	remoteAddCommand.call();

	FetchCommand fetchCommand = git.fetch();
	fetchCommand.call();

	CheckoutCommand checkoutCommand = git.checkout();
	checkoutCommand.setCreateBranch(true).setName("master").setUpstreamMode(SetupUpstreamMode.TRACK)
		.setStartPoint("origin/master").call();
    }
}

==> gsbot-maven-plugin-1.0.1-sources.jar: io/gupshup/maven/Add.java <==
/*
 * Copyright 2017 Gupshup.io. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.gupshup.maven;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.List;
import java.util.Properties;

import org.apache.maven.model.Resource;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;
import org.eclipse.jgit.api.CheckoutCommand;
import org.eclipse.jgit.api.CreateBranchCommand.SetupUpstreamMode;
import org.eclipse.jgit.api.FetchCommand;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.InitCommand;
import org.eclipse.jgit.api.RemoteAddCommand;
import org.eclipse.jgit.transport.URIish;
import org.json.JSONObject;

import io.gupshup.maven.utils.Constants;
import io.gupshup.maven.utils.Constants.SMAPIConsts;
import io.gupshup.maven.utils.SMAPIWrapper;

/**
 * @author Abhishek Nama
 */
@Mojo(name = "add", threadSafe = true, defaultPhase = LifecyclePhase.INITIALIZE, requiresDependencyResolution = ResolutionScope.COMPILE_PLUS_RUNTIME)
public class Add extends AbstractMojo {
    @Parameter(defaultValue = "${project}", readonly = true, required = true)
    private MavenProject project;

    @Parameter(property = "apiKey", required = true)
    private String apiKey;

    @Parameter(property = "botName", required = true)
    private String botName;

    @Override
    public void execute() throws MojoExecutionException {
	try {
	    List<?> resources = project.getResources();
	    if (resources.size() < 1)
		throw new MojoExecutionException("Project resources not found.");

	    Resource resource = (Resource) resources.get(0);
	    File gsCfgPropsFile = new File(resource.getDirectory() + File.separator + Constants.BOT_PROPERTIES);
	    if (!gsCfgPropsFile.exists())
		throw new MojoExecutionException("gsbot-cfg.properties not found in src" + File.separator + "main"
			+ File.separator + "resources");

	    Properties gsCfgProps = new Properties();
	    gsCfgProps.load(new FileInputStream(gsCfgPropsFile));

	    getLog().info("");
	    getLog().info("------------------------------------------------------------------------ ");
	    getLog().info("Adding bot " + botName + " for " + project.getName());
	    getLog().info("------------------------------------------------------------------------ ");
	    getLog().info("");
	    
	    /*getLog().info("Checking bot type...");

	    JSONObject getBotMetaResp = SMAPIWrapper.getBotMeta(apiKey, botName);
	    if (getBotMetaResp.getInt("status") != 200)
		throw new MojoExecutionException(getBotMetaResp.getString("body"));

	    JSONObject getBotMetaRespBody = new JSONObject(getBotMetaResp.getString("body"));
	    if (!(getBotMetaRespBody.has("function") && getBotMetaRespBody.getJSONObject("function").has("lang")
		    && getBotMetaRespBody.getJSONObject("function").getString("lang").equals("java"))) {
		throw new MojoExecutionException("Only java bots are supported.");
	    }

	    getLog().info("Bot type is Java.");
	    getLog().info("");*/

	    getLog().info("Fetching bot details...");

	    JSONObject getBotSpecResp = SMAPIWrapper.getBotSpec(apiKey, botName);
	    if (getBotSpecResp.getInt("status") != 200)
		throw new MojoExecutionException(getBotSpecResp.getString("body"));

	    String botKey = new JSONObject(getBotSpecResp.getString("body")).getString("uuid");

	    getLog().info("Configuring gsbot-cfg.properties");

	    gsCfgProps.setProperty("gupshup.dev.apikey", apiKey);
	    gsCfgProps.setProperty("gupshup.bot.name", botName);
	    gsCfgProps.setProperty("gupshup.bot.key", botKey);
	    gsCfgProps.store(new FileOutputStream(gsCfgPropsFile), null);

	    // Creating bot repository if not yet created.
	    SMAPIWrapper.createGit(botKey, apiKey);

	    getLog().info("Synching with bot repository.");

	    String userId = SMAPIWrapper.getUserId(apiKey);
	    if (userId != null) {
		String gitUrl = SMAPIConsts.GIT_URL.replace("__USERID", userId).replace("__APIKEY", apiKey)
			.replace("__BOTKEY", botKey);

		try {
		    cloneBotRepo(gitUrl);
		} catch (Exception e) {
		    getLog().info("");
		    if (e.getMessage() != null)
			getLog().warn(e.getMessage());
		    else
			getLog().warn("Failed to sync with bot repository.");
		}
	    } else {
		getLog().info("");
		getLog().error("Failed to sync with bot repository.");
	    }

	    getLog().info("");
	    getLog().info("Bot added successfully!");
	    getLog().info("");
	} catch (Exception e) {
	    throw new MojoExecutionException("", e);
	}
    }

    private void cloneBotRepo(String gitUrl) throws Exception {
	InitCommand initCommand = Git.init().setDirectory(project.getBasedir());
	Git git = initCommand.call();

	RemoteAddCommand remoteAddCommand = git.remoteAdd();
	remoteAddCommand.setName("origin");
	remoteAddCommand.setUri(new URIish(gitUrl));
	remoteAddCommand.call();

	FetchCommand fetchCommand = git.fetch();
	fetchCommand.call();

	CheckoutCommand checkoutCommand = git.checkout();
	checkoutCommand.setCreateBranch(true).setName("master").setUpstreamMode(SetupUpstreamMode.TRACK)
		.setStartPoint("origin/master").call();
    }
}
